###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
#
#

__all__ = ['UploadMultipart', 'TacticUploadException']

import socket
import base64

try:
    import urlparse
except:
    from urllib import parse as urlparse

try:
    import httplib
except:
    from http import client as httplib


import os, sys

class TacticUploadException(Exception):
    pass

class UploadMultipart(object):
    '''Handles the multipart content type for uploading files.  Will break up
    a file into chunks and upload separately for huge files'''

    def __init__(self):
        self.tries = 0
        self.chunk_size = 10*1024*1024
        self.ticket = None
        self.subdir = None

        self.server_url = None

        self.offset = 0


    def set_offset(self, offset):
        self.offset = offset


    def set_upload_server(self, server_url):
        self.server_url = server_url


    def set_chunk_size(self, size):
        '''set the chunk size of each upload'''
        self.chunk_size = size

    def set_ticket(self, ticket):
        '''set the ticket for security'''
        self.ticket = ticket

    def set_subdir(self, subdir):
        self.subdir = subdir


    def execute(self, path):
        assert self.server_url
        #f = open(path, 'rb')
        import codecs
        f = codecs.open(path, 'rb')

        if self.offset:
            f.seek(self.offset * self.chunk_size)
        else:
            self.offset = 0

        while 1:
            buffer = f.read(self.chunk_size)
            if not buffer:
                break

            if self.offset == 0:
                action = "create"
            else:
                action = "append"

            fields = [
                ("ajax", "true"),
                ("action", action),
            ]
            if self.ticket:
                fields.append(("ticket", self.ticket))
                fields.append(("login_ticket", self.ticket))
                basename = os.path.basename(path)
                from json import dumps as jsondumps

                # Workaround for python inside Maya, maya.Output has no sys.stdout.encoding property
                try:
                    if getattr(sys.stdout, "encoding", None) is not None and sys.stdout.encoding:
                        basename = basename.decode(sys.stdout.encoding)
                    else:
                        import locale
                        basename = basename.decode(locale.getpreferredencoding())
                except AttributeError:
                    # Python3 has no decode method on strings objects
                    pass

                basename = jsondumps(basename)
                basename = basename.strip('"')
                # the first index begins at 0
                fields.append(("file_name0", basename))

            if self.subdir:
                fields.append(("subdir", self.subdir))

            files = [("file", path, buffer)]
            (status, reason, content) = self.upload(self.server_url, fields, files)

            if reason != "OK":
                raise TacticUploadException("Upload of '%s' failed: %s %s" % (path, status, reason))

            self.offset += 1

        f.close()



    def upload(self, url, fields, files):

        try:
            ret_value = self.posturl(url, fields, files)

            if ret_value[0] != 200:
                raise Exception(ret_value[1])

            return ret_value

        except Exception as e:
            print("Error: ", e)

            # retry about 5 times
            print("... trying again")
            self.tries += 1
            if self.tries < 5:
                self.upload(url, fields, files)

        finally:
            self.tries = 0



    # Repurposed from:
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/146306

    def posturl(self, url, fields, files):
        #print("URL ", url)
        urlparts = urlparse.urlsplit(url)
        protocol = urlparts[0]
 
        return self.post_multipart(urlparts[1], urlparts[2], fields,files, protocol)
                


    def post_multipart(self, host, selector, fields, files, protocol):
        '''
        Post fields and files to an http host as multipart/form-data.
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files.dirk.noteboom@sympatico.ca
        '''
        content_type, body = self.encode_multipart_formdata(fields, files)
        if protocol == 'https':
            h = httplib.HTTPSConnection(host)  
        else:
            h = httplib.HTTPConnection(host)  
        headers = {
            'User-Agent': 'Tactic Client',
            'Content-Type': content_type
            }

        # prevent upgrading the method + url in the httplib module to turn it 
        # into a unicode string before sending the request
        selector = str(selector)
        h.request('POST', selector, body, headers)
        res = h.getresponse()
        return res.status, res.reason, res.read()    


    def encode_multipart_formdata(self, fields, files):
        '''
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files.
        Return (content_type, body) ready for httplib.HTTPConnection instance
        '''
        BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_---$---'
        CRLF = '\r\n'
        L = []

        mode = "base64"
        if mode != "base64":
            CRLF = CRLF.encode("UTF8")

        try:
            from cStringIO import StringIO as Buffer
        except:
            if mode == "base64":
                from io import StringIO as Buffer
            else:
                from io import BytesIO as Buffer


        import sys
        for (key, value) in fields:
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"' % key)
            L.append('')
            L.append(value)
        for (key, filename, value) in files:
            #print("len of value: ", len(value))
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
            L.append('')


            if mode == "base64":
                # put in a fake header to show that it is base64 to the server
                L.append("data:xyz/xyz;base64,")
                L.append(base64.b64encode(value))
            else:
                L.append(value)
        L.append('--' + BOUNDARY + '--')
        L.append('')

        M = []
        for l in L:
            try:
                if mode == "binary":
                    l = l.encode("UTF8")
                    #l = bytes(l)
                else:
                    l = l.decode()
            except UnicodeDecodeError as e:
                pass
            except AttributeError as e:
                pass
            M.append(l)
            M.append(CRLF)

        # This fails
        #body = "".join(M)

        buf = Buffer()
        buf.writelines(M)
        body = buf.getvalue()
        #print("len of body: ", len(body), type(body))

        content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
        return content_type, body 








###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['Pipeline']

IMPORT_ERROR = None
try:
    from xml.dom.ext.reader import Sax2
    from xml import xpath
except ImportError:
    IMPORT_ERROR =  "WARNING: pipeline.py in client api requires PyXML'"


class Pipeline(object):
    '''class that stores the data structure of the a pipeline.  Internally,
    this is stored as a linked list'''

    def __init__(self, pipeline_xml):
        if IMPORT_ERROR:
            print(IMPORT_ERROR)
            return
        self.doc = Sax2.FromXml(pipeline_xml)


    def get_first_process_name(self):
        # for now, just assume the first process
        nodes = xpath.Evaluate("/pipeline/process", self.doc)
        node = nodes[0]
        return node.getAttribute('name')


    def get_process_info(self, process_name):
        processes = xpath.Evaluate("/pipeline/process[@name='%s']" % process_name, self.doc)
        if not processes:
            return {}
        process = processes[0]

        #print("get_process_info: ", process_name)
        return process_name

        



    def get_output_process_names(self, process_name):
        attrs = xpath.Evaluate("/pipeline/connect[@from='%s']/@to" % process_name, self.doc)
        return [x.value for x in attrs]


    def get_input_process_names(self, process_name):

        attrs = xpath.Evaluate("/pipeline/connect[@to='%s']/@from" % process_name, self.doc)
        return [x.value for x in attrs]


    def get_handler_class(self, process_name):
        nodes = xpath.Evaluate("/pipeline/process[@name='%s']/action" % process_name, self.doc)
        if not nodes:
            return ""

        action = nodes[0]
        action_class = action.getAttribute("class")
        return action_class


    def get_action_options(self, process_name):
        options = {}
        nodes = xpath.Evaluate("/pipeline/process[@name='%s']/action" % process_name, self.doc)
        if not nodes:
            return options

        action_node = nodes[0]
        nodes = action_node.childNodes
        for node in nodes:
            name = node.nodeName
            if name == "#text":
                continue
            value = self._get_node_value(node)
            options[name] = value

        return options



    def _get_node_value(cls, node):
        '''Gets the value of a node.  This value is often the first child
        of the node'''
        value = node.nodeValue
        if value == None:
            if node.firstChild == None:
                value = ""
            else:
                value = node.firstChild.nodeValue

        return value
    _get_node_value = classmethod(_get_node_value)




###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


# Python3 - This is not support ... not even sure if we need this anymore

import os, sys
if sys.version_info[0] < 3:
    path = __file__
    path, file = os.path.split(path)
    path = '%s/tactic.zip' % path
    if path not in sys.path:
        sys.path.insert(0, path)

from .tactic_server_stub import *
from .cgapp import *







###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

from handler import *



###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys, getopt

from tactic_client_lib import TacticServerStub

SEARCH_TYPE = "prod/shot"

def main(args, login=None):
    # USAGE: checkin_shot.py <shot_code> <context> <path>
    shot_code = args[0]
    context = args[1]
    file_path = args[2]

    server = TacticServerStub(login)
    search_key = server.build_search_key(SEARCH_TYPE, shot_code)

    # do the actual work
    server.start("Checked in file [%s]" % file_path)
    try:
        # upload the file
        #server.upload_file(file_path)

        # checkin the uploaded file
        result = server.simple_checkin(search_key, context, file_path, mode='upload')
    except:
        server.abort()
        raise
    else:
        server.finish()



if __name__ == '__main__':
    
    executable = sys.argv[0]
    login = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "l:h", ["login=","help"])
    except getopt.error as e:
        print(e)
        sys.exit(2)
    # process options
    for o, a in opts:
        if o in ("-l", "--login"):
            login = a
            print("")
        if o in ("-h", "--help"):
            print("python checkin_shot.py -l <tactic_login> <code> <context> <file_path>")
            print("python checkin_shot.py -l admin S0001 anim C:/shot_S0001.ma")
   
    if len(args) != 3:
        print("python checkin_shot.py -l <tactic_login> <code> <context> <file_path>")
        print("python checkin_shot.py -l admin S0001 anim C:/shot_S0001.ma")
        sys.exit(2)
    main(args, login=login)




#!/usr/bin/python 
###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


import tacticenv
from tactic_client_lib import TacticServerStub

import unittest

class ExpressionApiTest(unittest.TestCase):

    def setUp(self):
        pass

    def test_all(self):

        self.server = TacticServerStub()
        project_code = "unittest"
        self.server.set_project(project_code)

        self.server.start("Expression Test")
        try:
            self._setup()


            self._test_expression()

        except:
            self.server.abort()
            raise
        else:
            self.server.abort()


    def _setup(self):

        city_data = {
            'code': 'los_angeles'
        }

        search_type = "unittest/person"
        self.persons = []
        for i in range(0,4):
            data = {
                'name_first': 'person%s' % i,
                'name_last': 'Test',
                'city_code': 'los_angeles',
                'age': '25'
            }
            person = self.server.insert(search_type, data)
            self.persons.append( person )


    def _test_expression(self):

        # get the people sobjects
        expr = "@SOBJECT(unittest/person)"
        result = self.server.eval(expr)
        self.assertEquals(4, len(result))
        self.assertEquals("los_angeles", result[0].get("city_code") )

        # get a single person
        expr = "@SOBJECT(unittest/person)"
        result = self.server.eval(expr, single=True)
        self.assertEquals("los_angeles", result.get('city_code'))

        # get the first_name
        expr = "@GET(unittest/person.name_first)"
        names = self.server.eval(expr)
        self.assertEquals(len(names), 4)
        #self.assertEquals("person0", names[0])
        #self.assertEquals("person1", names[1])
 
 
        # count the number of people
        expr = "@COUNT(unittest/person)"
        count = self.server.eval(expr)
        self.assertEquals(4, count)

        # get the age of a person
        expr = "@GET(unittest/person.age)"
        age = self.server.eval(expr, self.persons[0], single=True)
        self.assertEquals(25, age)

      
        return
       

if __name__ == "__main__":
    unittest.main()






###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['TestHandler', 'TestNextProcessHandler']

from tactic_client_lib.interpreter import Handler


class TestHandler(Handler):
    def __init__(self):
        self.company = None
        super(TestHandler, self).__init__()

    def execute(self):
        # set the output for this process if it is model
        if self.get_process_name() == "model":
            self.set_output_value("file", "test.txt")

        self.company = self.get_package_value('company')
        self.set_status("complete")



class TestNextProcessHandler(TestHandler):

    def execute(self):
        self.add_next_process("extra1")
        self.add_next_process("extra2")

        self.set_status("complete")







#########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

from application import *
from node_data import *
from tactic_node_util import *
from session import *



###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


# ALPHA: do not use

__all__ = ['CGApp']

import types

from tactic_client_lib import TacticServerStub


class CGAppException(Exception):
    pass
    


class CGApp(object):

    def __init__(self):

        # need to set up the environment (don't fight it for now!)
        from pyasm.application.common import BaseAppInfo

        from pyasm.application.common import AppEnvironment
        from pyasm.application.maya import Maya85
        self.env = AppEnvironment.get()
        self.env.set_tmpdir("C:/Temp")

        self.app = Maya85()
        self.env.set_app(self.app)


    #
    # High level methods
    #
    def checkin(self, tactic_node, search_key, context):
        '''Standard checkin mechanism.  This is a sample checkin mechanism
        that is full featured enough to handle many situations.

        @params:
          tactic_node: the tactic node to be checkedin
          search_key: the search key to check into
          context: the context to check into

        @return
          snapshot: a dictionary representing the final snapshot
          
        
        '''
        server = TacticServerStub.get()

        # verify that this is not a reference!!!


        # create a snapshot
        snapshot = server.create_snapshot(search_key, context)

        # find all of the dependencies
        dependent_nodes = self.get_dependent_nodes(tactic_node)
        for node in dependent_nodes:
            # find out if there is a node there is a file associated with
            # this tactic node
            files = self.get_dependent_references(node)
            print(files)


        # update the tactic node with the latest snapshot data.  Also, update
        # the tactic node name as it likely has changed
        node_data = self.set_introspect_data(tactic_node, snapshot)
        tactic_node = node_data.get_app_node_name()


        # add the files to the snapshot
        handler = BaseFileExtractionHandler(tactic_node)
        paths = handler.execute()
        #paths = self.extract_to_files(top_node)
        for path in paths:
            print("path: ", path)
            server.add_file(snapshot.get("code"), path, mode='upload')

        return snapshot


    def load(self, search_key, context, version=-1, file_type='main', mode='reference',namespace=''):
        '''Generic loading function used by TACTIC interface by default

        @params:
        search_key: the search key of the sobject to be loaded
        context: the context of the snapshot to be loaded
        version: the version of the snapshot to be loaded
        file_type: the specific file in the snapshot to be loaded

        mode: reference|import|open: the mode in which to bring the file in
        
        
        '''
        server = TacticServerStub.get()

        # FIXME: do we really need to make 2 calls?
        snapshot = server.get_snapshot(search_key, context=context, version=version)
        if not snapshot:
            raise CGAppException("No snapshot found for [%s]" % search_key)


        paths = server.get_paths(search_key, context, version, file_type)
        web_paths = paths.get("web_paths")

        for path in web_paths:
            to_path = self.download(path)
            self.load_file(to_path, namespace=namespace, mode=mode)


        # FIXME: if the instance already exists, it will be auto renmaed by
        # the application ... this gets tricky to discover
        snapshot_code = snapshot.get('code')
        if namespace:
            tactic_node = "%s:tactic_%s" % (namespace, snapshot_code)
        else:
            tactic_node = "tactic_%s" % snapshot_code
        return tactic_node



    #
    # Node methods
    #


    def get_tactic_nodes(self, top_node=None):
        return self.app.get_tactic_nodes()


    def is_tactic_node(self, node):
        '''determines if a node is a tactic node or not

        @param
        node_name: name of the maya node

        @return
        True/False

        '''
        return self.app.is_tactic_node(node)




    #
    # Dependency methods
    #
    def get_dependent_nodes(self, tactic_node):
        '''find all of the dependent TACTIC nodes'''
        return ['chr001']

    def get_dependent_references(self, tactic_node):
        '''determine all of the files that are part of this tactic_node'''
        return ['file1.ma', 'file2.ma']


    def is_node_repo(self, tactic_node):
        '''determines is a TACTIC node is local or from the repo'''
        pass

    def get_dependent_textures(self, tactic_node):
        '''determine all of the files that are part of this tactic_node'''
        return {
            'file1': '/home/apache/whatever.png',
            'file2': '/home/apache/whatever2.png'
        }


    #
    # Loading methods
    #
    def load_file(self, path, mode='reference', namespace=''):
        '''Load a file in the session
        
        @params:
          path: the full path of the file to be loaded into the session
          mode: reference|import|load
          namespace:
        '''
        if mode == 'reference':
            self.app.import_reference(path, namespace=namespace)
        elif mode == 'load':
            self.app.load(path)
        elif mode == 'import':
            self.app.import_file(path, namespace=namespace)
        else:
            raise CGAppException("Load mode [%s] not supported" % mode)



    #
    # Extraction methods
    #
    def extract_to_files(self, tactic_node, dir=None):

        # first get the node to be extracted
        top_node = self.app.get_parent( tactic_node )
        if not top_node:
            top_node = tactic_node

        tmp_dir = self.env.get_tmpdir()
        file_type = 'mayaAscii'
        preserve_ref = True

        context = 'temp_context'    # Should not be necessary
        filename = '%s.ma' % tactic_node
        instance = 'whatever'       # Should not be necessary


        # use the old export node method to extract
        path = self.app.export_node(top_node, context, tmp_dir,
            type=file_type, preserve_ref=preserve_ref, filename=filename,
            instance=instance)

        return [path]


    #
    # Introspect methods
    #
    def introspect(self):
        '''introspects the session and updates the database'''
        pass

    def set_introspect_data(self, tactic_node, snapshot):
        '''adds intropection data to a tactic node
        
        @params:
          tactic_node: the node in the maya session to add the
            introspection data
          snapshot: the snapshot from which to add the information

        @return
        NodeData object
        '''
        #if not self.is_tactic_node(tactic_node):
        #    raise CGAppException("Node [%s] is not a Tactic node" % tactic_node)

        # add the attr if it doesn't exist
        #self.app.add_attr(tactic_node, "tacticNodeData")

        # rename to tactic_<snapshot_code>
        snapshot_code = snapshot.get("code")
        new_tactic_node = "tactic_%s" % snapshot_code

        tactic_node = self.app.rename_node(tactic_node, new_tactic_node)
        assert tactic_node == new_tactic_node

        snapshot_code = snapshot.get('code')

        from pyasm.application.common import NodeData
        node_data = NodeData(tactic_node, self.app)
        node_data.create()
        node_data.set_attr("snapshot", "code", snapshot_code)
        node_data.commit()

        return node_data





    #
    # Common methods
    #
    def download(self, url, to_dir=''):
        return self.env.download(url, to_dir)





class BaseFileExtractionHandler(object):

    def __init__(self, tactic_node):
        self.tactic_node = tactic_node

        from pyasm.application.common import AppEnvironment
        self.env = AppEnvironment.get()
        self.app = self.env.get_app()

    def execute(self):

        # first get the node to be extracted
        top_node = self.app.get_parent( self.tactic_node )
        if not top_node:
            top_node = self.tactic_node

        tmp_dir = self.env.get_tmpdir()
        file_type = 'mayaAscii'
        preserve_ref = True

        context = 'temp_context'    # Should not be necessary
        filename = '%s.ma' % self.tactic_node
        instance = 'whatever'       # Should not be necessary


        # use the old export node method to extract
        path = self.app.export_node(top_node, context, tmp_dir,
            type=file_type, preserve_ref=preserve_ref, filename=filename,
            instance=instance)

        return [path]








###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys, getopt

from tactic_client_lib import TacticServerStub, TacticApiException



def main(args, src_snapshot_code=None, src_search_type=None, src_search_code=None, src_context=None, src_version=None,\
        tgt_context=None, tgt_version=None, file_path=None):

    if len(args) == 3:
        # assume code and file path are equivalent
        search_type = args[0]
        code = args[1]
        bin_info = args[2]
    elif len(args) == 4:
        search_type = args[0]
        code = args[1]
        bin_info = args[2]
        file_path = args[3]
    server = TacticServerStub()

    if file_path:
        description = "Dailies Submission [%s] to [%s] - [%s]" % (file_path, search_type, code)
    elif src_snapshot_code:
        description = "Dailies Submission internal reference [%s] to [%s] - [%s]" \
            % (src_snapshot_code, search_type, code)
    else:
        description = "Dailies Submission internal reference [%s|%s] to [%s] - [%s]" \
            % (src_search_type, src_search_code, search_type, code)
    server.start(title='Dailies Submission', description=description )
    try:
        parts = bin_info.split('|')
        if len(parts) == 3:
            bin_code, type, label = parts[0], parts[1], parts[2]
            bins = server.query('prod/bin', [('code', bin_code), ('label', label), ('type', type)])
        elif len(parts) == 2:
            bin_code, type = parts[0], parts[1]
            bins = server.query('prod/bin', [('code', bin_code), ('type', type)])
        if not bins:
            raise TacticApiException("Bin code [%s], type [%s], label [%s] not found in system " % (bin_code, type, label))
    
        
        search_key = server.build_search_key(search_type, code)
        sobject = server.query(search_type, [("code", code)])
        if not sobject:
            raise TacticApiException("SObject [%s] with code [%s] not found" %(search_type, code))

        sobject_id = sobject[0].get('id')
        # add a new submission
        full_search_type = server.build_search_type(search_type)
        submit_data = {'search_type': full_search_type,\
                        'search_id': sobject_id,\
                        'description': "Client Api Submission: [%s]" %code\
                        }
        if tgt_context and tgt_version != None:
            submit_data['context'] = tgt_context
            if tgt_version in ['0', 0 ,'-1', -1]:
                tgt_snapshot = server.get_snapshot(search_key, tgt_context, version=tgt_version)
                if not tgt_snapshot:
                    print "tgt_context [%s] and tgt_version [%s] for this sobject cannot be found" %(tgt_context, tgt_version)
                else: 
                    tgt_version = tgt_snapshot.get('version')
            submit_data['version'] = tgt_version

        src_snapshot = None
        if src_snapshot_code:
            src_snapshot = server.query('sthpw/snapshot',filters=[('code', src_snapshot_code)], single=True)
            submit_data['context'] = src_snapshot.get('context')
            submit_data['version'] = src_snapshot.get('version')

        submission = server.insert('prod/submission', submit_data)
        # add to bin
        submit_in_bin_data = { 'bin_id': bins[0].get('id'), \
                                'submission_id': submission.get('id')\
                                }
        server.insert('prod/submission_in_bin', submit_in_bin_data)
        
        print "New submission [%s] created for [%s].\n" %(submission.get('__search_key__'), full_search_type)
         
        if file_path:
            #NOTE: use the upload method instead of copy or move 
            server.simple_checkin(submission.get('__search_key__'), 'publish', file_path, \
                    snapshot_type='submission', description='Client API dailies submission', mode='upload')
        else:
            if src_snapshot:
               
                pass 
            elif src_search_type:
                src_search_key = server.build_search_key(src_search_type, src_search_code)
            else:
                src_search_key = search_key

            if not src_snapshot:
                src_snapshot = server.get_snapshot(src_search_key, context=src_context, version=src_version)
        
            if src_snapshot:
                snapshot_code = src_snapshot.get('code')
                
                # build a new snapshot
                
                submit_snapshot = server.create_snapshot(submission.get('__search_key__'), \
                        "publish", description='Client API dailies submission', \
                        snapshot_type='submission')

                # add dependency
                server.add_dependency_by_code(submit_snapshot.get('code'), snapshot_code)
            else:
                raise TacticApiException("No snapshot found for [%s] with context [%s] and version [%s]."\
                        %(src_search_key, src_context, src_version))
                return
        
        

    except Exception, e:
        server.abort()
        print "ERROR: ", e.__str__()
    else:
        server.finish()


if __name__ == '__main__':
    
    executable = sys.argv[0]
    src_search_type = None
    src_search_code = None
    src_context = None
    src_version = None
    tgt_context = None
    tgt_version = None
    file_path = None
    try:
        from optparse import OptionParser
        parser = OptionParser()
        parser.add_option("--ssn", help="Source snapshot code")
        parser.add_option("--sst", help="Source search type")
        parser.add_option("--ssc", help="Source search code")
        parser.add_option("--sc", help="Source context")
        parser.add_option("--sv", help="Source version")
        parser.add_option("--tc", help="Target context")
        parser.add_option("--tv", help="Target version")
      
        opts, args = parser.parse_args()
    except getopt.error, msg:
        print msg
        sys.exit(2)
    # process options
    
    src_search_type = opts.sst
    src_search_code = opts.ssc
    src_snapshot_code = opts.ssn
    src_context  = opts.sc
    src_version  = opts.sv
    tgt_context = opts.tc
    tgt_version = opts.tv

    usage = "USAGE: submit_dailies.py <search_type> <code> <bin_info> [path] [--sst search_type] [--ssc search_code] [--sc context] [--sv version] [--tc context] [--tv version]\n"
    usage += "example1: python submit_dailies.py prod/shot HT001_010 \"2008-10-10|dailies|review\" --tc anim --tv -1 \"C:/test/berry.jpg\"\n"
    usage += "example2: python submit_dailies.py prod/shot HT001_010 \"2008-10-10|dailies\" --sst prod/asset --ssc char001 --sc rig --sv 5\n'"
    usage += "example2: python submit_dailies.py prod/shot HT001_010 \"2008-10-10|dailies\"--ssn 101BAR\n'"

    # TODO: lots of assumptions here

    if len(args) == 3:
        # assume code and file path are equivalent
        search_type = args[0]
        code = args[1]
        bin_info = args[2]
        if (not src_context or not src_version) and not src_snapshot_code:
            print "source context --sc and source version --sv required if no <file_path> is provided."
            sys.exit(2)

    elif len(args) == 4:
        search_type = args[0]
        code = args[1]
        bin_info = args[2]
        file_path = args[3]
    
    else:
        print usage
        sys.exit(2)

    parts = bin_info.split('|')
    if len(parts) not in [2, 3]:
        print "The format for bin info is either <bin_code>|<type> or <bin_code>|<type>|<label>."
        print "e.g. 2008-10-20|dailies or 2008-10-20|dailies|review."
        sys.exit(2)

    main(args, src_snapshot_code=src_snapshot_code, src_search_type=src_search_type, src_search_code=src_search_code, \
            src_context=src_context, src_version=src_version,\
            tgt_context=tgt_context, tgt_version=tgt_version, file_path=file_path)




###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['Application', 'AppException']

import sys, types

from node_data import NodeData
from app_environment import AppException


class Application(object):
    '''base class defining interface for 3D applications'''

    def __init__(self, init=True):

        self.buffer_flag = False
        self.buffer = None

        self.verbose = False


    def message(self, message):
        print message


    def start_buffer(self):
        '''starts a buffer which redirects commands to a buffer'''
        self.buffer_flag = True
        self.buffer = []

    def end_buffer(self):
        self.buffer_flag = False

    def get_buffer(self):
        return self.buffer

    def set_verbose(self, flag=True):
        self.verbose = flag




    def cleanup(self):
        '''Note: necessary in Houdini'''
        pass


    def get_node_data(self, node_name):
        return NodeData(node_name)

    # Common operations
    #   These abstract and interface to appplication version so that
    # implementations can made.  As few basic operations as possible
    # into the application are defined to simplify porting.
    def is_tactic_node(self, node):
        raise AppException("Function: 'is_tactic_node()' must be overriden")

    def set_project(self, project_dir):
        raise AppException("Function: 'set_project()' must be overriden")

    def get_project(self):
        raise AppException("Function: 'get_project()' must be overriden")

    def get_var(self):
        raise AppException("Function: 'get_var()' must be overriden")

    def get_node_type(self, node_name):
        raise AppException("Function: 'get_node_type()' must be overriden")
    
    def get_children(self, node_name):
        pass
    
    # action functions
    def set_attr(self, node, attr, value, attr_type=""):
        pass

    def select(self, node):
        raise AppException("Function: 'select()' must be overriden")

    def select_add(self, node):
        pass
    
    def select_none(self):
        pass

    def select_restore(self, nodes):
        pass
        
    def select_hierarchy(self, node): 
        pass

    
    # interaction with files
    def import_file(self, path, namespace=":"):
        '''import a file into the session'''
        pass


    def import_reference(self, path, namespace=":"):
        '''load using renferences'''
        pass

    def replace_reference(self, node_name, path, top_reference=True):
        '''load using renferences'''
        pass



    def is_reference(self, instance):
        '''detect whether an instance is a reference or not'''
        pass

    def is_keyed(self, node_name, attr):
        '''detect whether an instance is keyed or not'''
        pass

    def import_static(self, buffer, node_name):
        '''import unkeyed values'''
        pass
        
    def import_anim(self, path, namespace=":"):
        '''load in animation'''
        pass


    def export_anim(self, path, namespace=":"):
        '''export the animation'''


    def save(self, path, file_type=None):
        '''save the file'''
        raise AppException("Function: 'save()' must be overriden")

    def load(self, path):
        '''load the file'''
        raise AppException("Function: 'load()' must be overriden")

    def rename(self, path):
        '''rename the file'''
        # this is harmless if it doesn't do anything
        pass


    def export_node(self, node_name, context, dir=None, filename="", preserve_ref=None):
        '''export a top node'''
        pass

    def save_node(self, node_name, dir=None):
        '''use file/save for a particular node (saves whole file instead
        of using export'''
        pass

    def get_file_path(self):
        '''returns the path of the last save filename'''
        raise AppException("Function: 'get_file_path()' must be overriden")



    # namespace commands
    def set_namespace(self, namespace=":"):
        pass

    def add_namespace(self, namespace):
        pass

    def remove_namespace(self, namespace):
        pass

    def namespace_exists(self, namespace):
        pass

    def get_namespace_info(self):
        pass


    def get_namespace_contents(self):
        '''retrieves the contents of the current namespace'''
        return []

    def get_all_namespaces(self):
        return []



    def rename_node(self, node_name, new_name):
        pass



    # set functions
    def get_sets(self):
        return []

    def is_set(self, node_name):
        if node_name in self.get_sets():
            return True
        else:
            return False


    def create_set(self, node_name):
        pass
        
    def add_to_set(self, set_name, node_name):
        pass

    def get_nodes_in_set(self, set_name):
        pass





    # information retrieval functions.  Requires an open Maya session
    def node_exists(self,node):
        pass

    def get_nodes_by_type(self, type):
        pass



    def get_selected_node(self):
        nodes = self.get_selected_nodes()
        if nodes:
            return nodes[0]
        else:
            return None


    def get_selected_nodes(self):
        pass


    def get_selected_top_nodes(self):
        raise AppException("Function: 'get_selected_top_nodes()' must be overriden")


    def get_top_nodes(self):
        raise AppException("Function: 'get_top_nodes()' must be overriden")



    def get_reference_nodes(self, top_node, recursive=False):
        '''gets all of the references under a single dag node'''
        return []


    def get_reference_path(self, node):
        pass




    def add_node(self, type, node_name, unique=False):
        '''return name of node added'''
        pass


    # attributes
    def add_attr(self, node, attribute, type="long"):
        pass


    def attr_exists(self, node, attribute):
        '''return True or False'''
        raise AppException("Function: 'attr_exists()' must be overriden")


    def get_attr(self, node, attribute):
        pass

    def get_attr_type(self, node, attribute):
        pass

    def get_all_attrs(self, node):
        pass

    def get_attr_default(self, node, attr):
        pass

    # layers
    def get_all_layers(self):
        return []


    def get_layer_nodes(self):
        '''gets all of the TACTIC nodes in a layer'''
        return []


    def set_user_environment(self, sandbox_dir, basename):
        '''gives the opportunity to let TACTIC set the user envrionment based
        on the sandbox dir.  It is safe not to use this for a given application
        '''
        pass




class AppException(Exception):
    '''Used by different applications for raising exceptions'''
    pass




###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['TacticNodeUtil']


import xmlrpclib, os, shutil

from node_data import NodeData


# importing 8.5 maya module
try:
    import maya
except:
    print("WARNING: module MayaApplication requires 'maya' module")


class TacticNodeUtil(object):
    '''This class encapsulates functionality needed interact with information
    to communicate with any application.'''

    def __init__(self):
        import maya.standalone
        maya.standalone.initialize( name='python' )
        import maya.cmds, maya.mel

        from tactic_client_lib.application.maya import Maya
        self.app = Maya()
     

    def is_tactic_node(self, node_name):
        '''Determines whether a particular node is a tactic node'''
        pass

   

    def get_all_tactic_nodes(self):
        '''Gets all of the Tactic nodes in the session
        @return
        a list of all tactic node names
        '''

        tactic_nodes = []
        nodes = self.app.get_nodes_by_name("tactic_*")

        # go through each node an make sure that the attribute
        # exists
        for node in nodes:
            if self.is_tactic_node(node):
                tactic_nodes.append(node)

        return tactic_nodes


    def is_tactic_node(self, node_name):
        '''determines whether the given node is a tactic node
        '''
        attribute = "tacticNodeData"
        exists = self.mel("attributeExists %s %s" % (attribute, node_name))
        if not exists:
            return False
        else:
            return True



        
                

    #
    # action functions
    #
    def create_default_node(self, node_name, search_key=None, context=None):
        '''Creates a default template structure when loading a context
        that has no snapshots associated with it
        
        @params 
        node_name: the name of the node to be created
        search_key: a search_key representing the sobject that this node belongs
            to

        @return
        the name of the tactic node created
        '''

        # create the node
        type = "transform"
        node_name = self.app.add_node(node_name, type)

        # create the tactic node
        tactic_type = "transform"
        tactic_node_name = "tactic_%s" % node_name
        tactic_node_name = self.app.add_node(tactic_node_name, tactic_type)


        # add the tacticNodeData attribute and record the search type
        node_data = NodeData(tactic_node_name)
        if search_key:
            node_data.set_attr("ref", "search_key", search_key)
        if context:
            node_data.set_attr("ref", "context", context)
        node_data.commit()

        # attache the tactic node to the node
        self.mel("parent %s %s" % (tactic_node_name, node_name) )

        return tactic_node_name



    def add_snapshot_to_node(self, tactic_node_name, snapshot):
        snapshot_code = snapshot.get('code')
        search_type = snapshot.get('search_type')
        search_id = snapshot.get('search_id')
        context = snapshot.get('context')
        version = snapshot.get('version')
        revision = snapshot.get('revision')
        
        node_data = NodeData(tactic_node_name)
        node_data.set_attr("ref", "snapshot_code", snapshot_code)
        node_data.set_attr("ref", "search_type", search_type)
        node_data.set_attr("ref", "search_id", search_id)
        node_data.set_attr("ref", "context", context)
        node_data.set_attr("ref", "version", version)
        node_data.set_attr("ref", "revision", revision)

        node_data.commit()



    def get_search_key(self, tactic_node_name):
        '''gets the snapshot data on a particular tactic node
        <node>
          <ref context='model' version='3' search_type='prod/asset?project=bar' search_id='4'/>
        </node>

        '''
        node_data = NodeData(tactic_node_name)
        search_type = node_data.get_attr("ref", "search_key")
        return search_type

    def get_context(self, tactic_node_name):
        '''gets the snapshot data on a particular tactic node
        <node>
          <ref context='model' version='3' search_type='prod/asset?project=bar' search_id='4'/>
        </node>

        '''
        node_data = NodeData(tactic_node_name)
        search_type = node_data.get_attr("ref", "context")
        return search_type









    def mel(self, cmd, verbose=None):
        '''Excecute a mel command (TEMPORARY - use maya specific)'''
        return maya.mel.eval(cmd)






class SnapshotXml(object):
    '''Series of utilities to parse the snapshot xml'''
    
    def __init__(self, snapshot_xml):
        self.snapshot_xml = snapshot_xml








###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['callback']

from interpreter import PipelineInterpreter

import cgi

class ClientCallbackException(Exception):
    pass


class BaseClientCbk(object):
    def set_ticket(self, ticket):
        self.ticket = ticket

    def set_options(self, options):
        self.options = options

    def get_option(self, name):
        return self.options.get(name)


    def _execute(self):

        # get the server name
        server_name = self.get_option("server_name")
        if not server_name:
            raise ClientCallbackException("No [server_name] option delivered to callback")
        server_name = server_name[0]

        # get the project
        project_code = self.get_option("project_code")
        if not project_code:
            raise ClientCallbackException("No [project_code] option delivered to callback")
        project_code = project_code[0]

        # the entire pipeline is run in a TacticServer Transaction
        from tactic_client_lib import TacticServerStub
        self.server = TacticServerStub()
        self.server.set_server(server_name)
        self.server.set_ticket(self.ticket)
        self.server.set_project(project_code)


        self.server.start("Pipeline checkin")
        try:
            self.execute()

        except Exception, e:
            self.server.abort()
            raise
        else:
            self.server.finish()


    def execute_pipeline(self, pipeline_xml):
        # execute the pipeline
        interpreter = PipelineInterpreter(pipeline_xml)
        interpreter.set_server(self.server)
        interpreter.set_package(self.options)
        interpreter.execute()





class ClientCbk(BaseClientCbk):
    '''This callback executes a pipeline based on certain input parameters
    '''
    def execute(self):
        # get the snapshot type passed in
        snapshot_type_code = self.get_option("snapshot_type")
        if not snapshot_type_code:
            raise ClientCallbackException("No [snapshot_type] option delivered to callback")

        snapshot_type_code = snapshot_type[0]
        search_type = "sthpw/snapshot_type"
        search_key = self.server.build_search_type(search_type, snapshot_type_code)
        # get the pipeline
        pipeline_xml = self.server.get_pipeline_xml(search_key)

        self.execute_pipeline(pipeline_xml)





class ClientLoadCbk(BaseClientCbk):
    '''This callback executes a pipeline based on certain input parameters
    '''
    def execute(self):

        search_keys = self.get_option('search_key')
        for search_key in search_keys:
            print "search_key: ", search_key

            # get the snapshot
            snapshot = self.server.get_by_search_key(search_key)

            # get the snapshot type from the snapshot
            snapshot_type_code = snapshot.get('snapshot_type')
            search_type = "sthpw/snapshot_type"
            search_key = self.server.build_search_type(search_type, snapshot_type_code)
            pipeline_xml = self.server.get_pipeline_xml(search_key)

            # execute the pipeline
            self.execute_pipeline(pipeline_xml)





#
# entry point
#
def callback(ticket, callback_class, options):
    # NOTE: cgi.parse_qs creates arrays for *all* values
    options = cgi.parse_qs(options)
    print "options: ", options

    # get the callback, if there is one
    callback = eval("%s()" % callback_class)
    callback.set_ticket(ticket)
    callback.set_options(options)
    callback._execute()





#!/usr/bin/python 
###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['MayaTest']



import unittest
import xmlrpclib, sys, os, shutil

class MayaTest(unittest.TestCase):
    '''Test the basic level interaction to Maya'''

    def test_all(self):

        from tactic_client_lib.application.common import TacticNodeUtil
        self.util = TacticNodeUtil()
        from tactic_client_lib.application.maya import Maya
        self.app = Maya()

        self._test_nodes()
        self._test_attributes()
        self._test_file()



    def _test_nodes(self):

        # create a test node
        node_name = "test_node"
        created_name = self.app.add_node(node_name)
        self.assertEquals(node_name, created_name)

        # test to see whether it exists
        exists = self.app.node_exists(created_name)
        self.assertEquals(True, exists)

        # create a node of the same name
        created_name2 = self.app.add_node(node_name)
        self.assertEquals("test_node1", created_name2)

        # create a node that should be unique
        created_name3 = self.app.add_node(node_name, unique=True)
        self.assertEquals("test_node", created_name3)

        # get the nodes by name pattern
        pattern = "test_*"
        nodes = self.app.get_nodes_by_name(pattern)
        self.assertEquals(['test_node','test_node1'], nodes)
        





    def _test_attributes(self):
        '''tests various attribute functionality'''



    def _test_file(self):
        if os.name == "nt":
            dir = "C:/sthpw/temp"
        else:
            dir = "/tmp/sthpw/temp"
        if not os.path.exists(dir):
            os.makedirs(dir)

        path = '%s/test' % dir
        if os.path.exists(path):
            os.unlink(path)

        path = self.app.save(path)            
        self.assertEquals(True, os.path.exists(path))

        # remove the file
        if os.path.exists(path):
            os.unlink(path)



    def main():        
        import sys
        sys.path.insert(0, "C:/Program Files/Southpaw/Tactic1.9/src/client")
        try:
            unittest.main()
        except SystemExit:
            pass
    main = staticmethod(main)



if __name__ == "__main__":
    MayaTest.main()




#!/usr/bin/env python

###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys
import re

from tactic_client_lib import TacticServerStub, TacticApiException


def main(args):
    # USAGE: create_project.py <project_code> <project_title> <project_type> 
    project_code = args[0]
    project_title = args[1]
    project_type = args[2]

    assert project_type in ['prod','flash','simple','unittest']
    assert project_title

    regexs = r'^\d|\W'
    m = re.search(r'%s' % regexs, project_code) 
    if m:
        raise TacticApiException('<project_code> cannot contain special characters or start with a number.')

    server = TacticServerStub.get();
    # do the actual work
    server.start("Create Project", "Project creation for [%s] of type [%s]" % (project_code, project_type))
    try:
        args = {
        'project_code': project_code,
        'project_title': project_title,
        'project_type': project_type}
    
    
        
        class_name = "tactic.command.CreateProjectCmd";
        ret_val = server.execute_cmd(class_name, args=args);
        print(ret_val)
    except:
        server.abort()
        raise
    else:
        server.finish()



if __name__ == '__main__':
    
    executable = sys.argv[0]
    args = sys.argv[1:]
    if len(args) != 3:
        print("python create_project.py <project_code> <project_title> <project_type>")
        sys.exit(0)
    main(args)



# -------------------------------------------------------------------------------------------------------
#
#                        TACTIC Client API version bootstrap example script
#                        __________________________________________________
#
#
# == Summary ==
#
#        This script is an example of how to manage client API access to multiple versions of TACTIC, or
#    to manage client API access through upgrades and rollbacks of versions of a single TACTIC server.
#
#    What is required for this approach to work, and for this script to run successfully, are 2 things:
#
#    (1) that in each TACTIC install the VERSION file, at the very top of a TACTIC install, is copied
#        as a file named VERSION.txt into the src/context folder
#
#    (2) that every time a specific version of TACTIC is installed on the TACTIC server, a copy of
#        that version's src/client folder is pushed out to client workstations before a specific server
#        version install is switched to for the live TACTIC server
#
#        NOTE: this script assumes that these client-side copies of the client API package are located
#              and named in the following manner ...
#
#
#                C:\sthpw\client_api\tactic-2.6.0.rc01\client
#
#                C:\sthpw\client_api\tactic-2.6.0.rc02\client
#
#                C:\sthpw\client_api\tactic-2.6.0.v01\client
#
#
#              ... it is up to each studio to determine the location of, and the mechanism to push out
#              new versions of, the client API packages that need to be made available to client-side
#              running scripts.
#
#
# == The Concept ==
#
#        The overall concept here is to provide a way for pipeline scripts that need access to TACTIC
#    (and that will be running on client workstations) to be able to seamlessly load the appropriate
#    client API version that precisely matches the TACTIC version of the server that the script will be
#    accessing through the TACTIC client API.
#
#    To accomplish this, we establish a "bootstrap" function, see bootstrap_get_tactic_server_stub()
#    below, that is actually completely independent of TACTIC server version. This bootstrap function
#    is able to make an HTTP request to a specific TACTIC server (without using the TACTIC client API
#    which is TACTIC version dependent), in order to obtain the version of that TACTIC server. Once
#    the version is known, the bootstrap function dynamically appends the "client" folder of the matching
#    client API version that is accessible by the client workstation. The script then does the import
#    of the appropriate module class and then instantiates a tactic server stub instance against the
#    correct client API version and returns that server stub instance to the calling program.
#
#    So, this bootstrap function would need to become part of the standard studio python libraries
#    that would always be available to pipeline python scripts. All scripts in the pipeline that
#    needed to access TACTIC would then be designed to always use this bootstrap function to obtain
#    a connection to the given TACTIC server it would be running against.
#
# -------------------------------------------------------------------------------------------------------

import sys
import os
import getopt
import traceback
import six
from six.moves import input

try:
    import httplib
except:
    # Python3
    from http import client as httplib


def bootstrap_get_tactic_server_stub( tactic_server_str, project_code, login, password ):

    conn = httplib.HTTPConnection(tactic_server_str)
    conn.request("GET", "/context/VERSION.txt")

    response = conn.getresponse()
    # print response.status, response.reason

    data = response.read()
    tactic_version = data.strip()

    conn.close()

    print("")
    print("> Found TACTIC version '%s' running on server %s ..." % (tactic_version, tactic_server_str))
    print("")

    client_lib_install_path = "C:/sthpw/client_api/tactic-%s/client" % tactic_version

    print("  ... attempting to import 'TacticServerStub' from 'tactic_client_lib' located in:")
    print("")
    print("         %s" % client_lib_install_path)
    print("")

    try:
        sys.path.append( client_lib_install_path )
        from tactic_client_lib import TacticServerStub
        if not login or not password:
            print("")
            print("> Warning: login/password not provided, assuming the use of .tacticrc ticket ...")
            print("")
            tactic_server = TacticServerStub(server=tactic_server_str, project=project_code)
        else:
            tactic_server = TacticServerStub(
                                server=tactic_server_str,
                                project=project_code,
                                user=login,
                                password=password
                            )
    except:
        print("  ERROR: unable to access TACTIC client API for server version '%s'" % (tactic_version))
        if not login or not password:
            print("           (likely there is no .tacticrc ticket that exists)")

        print("")
        print("Exception found:")
        print('-' * 60)
        traceback.print_exc(file=sys.stdout)
        print('-' * 60)
        print("")
        tactic_server = None

    return tactic_server


def usage():

    script_name = os.path.basename(sys.argv[0])
    print("")
    print("  Usage: %s [options] <tactic_server_str> <project_code>" % (script_name))
    print("")
    print("         ____ OPTIONS ____")
    print("")
    print("         -h, --help ... print this usage message")
    print("")
    print("         -i, --interactive ... interactive user prompts to enter login & password")
    print("")
    print("         -l <login>, --login <login> ... user login")
    print("         -p <password>, --password <password> ... user password")
    print("")
    print("         NOTE: use only one of ... -i OR (-l <login> and -p <password>)")
    print("               if none of those options are used it is assumed to use an")
    print("               existing .tacticrc ticket")
    print("")
    print("    e.g. python %s -l dsmith -p mypassword 192.168.188.134 sample3d" % (script_name))
    print("")


if __name__ == '__main__':

    login = None
    password = None

    use_ticket = False
    prompt_user = False

    short_opts = "hil:p:"
    long_opts = [ "help", "interactive", "login=", "password=" ]

    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.error:
        usage()
        sys.exit(1)

    if len(args) != 2:
        usage()
        sys.exit(1)

    tactic_server_str = args[0]
    project_code = args[1]

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif opt in ("-i", "--interactive"):
            prompt_user = True
        elif opt in ("-l", "--login"):
            login = arg
        elif opt in ("-p", "--password"):
            password = arg

    if prompt_user:

        login = input("  Enter TACTIC login (just ENTER for current user): ")
        if not login:
            if sys.platform == 'win32':
                login = os.environ.get('USERNAME')
            else:
                login = os.environ.get('USER')

        password = input("  Enter TACTIC password for user '%s': " % login)
        if not password:
            print("")
            print("*** password required to run script ... no password entered ... aborting.")
            print("")
            sys.exit(1)

    else:
        if not (login and password) and not (not login and not password):
            usage()
            sys.exit(1)

    tactic_server = bootstrap_get_tactic_server_stub(tactic_server_str, project_code, login, password)

    if tactic_server:
        print("")
        print("ping: [%s]" % tactic_server.ping())
        print("")
    else:
        print("")
        print("*** NO connection to TACTIC server obtained ... unable to ping server ***")
        print("")





import sys, os, shutil, getopt

from tactic_client_lib import TacticServerStub

SEARCH_TYPE = "prod/plate"


    

def main(args, login=None):
    # USAGE: checkin_plates.py   
    code = args[0]
    file_range = args[1]
    pattern = args[2]
    # hard-coded for now
    context = 'publish'
    server = TacticServerStub(login)

    # do the actual work
    server.start("Checked in file group [%s]" % pattern)
    try:
        

        # checkin the uploaded file  
        filters = []
        filters.append(('code', code))
        results = server.query(SEARCH_TYPE, filters)
        # take the first one
        if results:
            id = results[0].get('id')
        else:
            print("Plate with code [%s] not found. Please insert an entry in the Plates tab first." % code)
            return
        search_key = server.build_search_key(SEARCH_TYPE, id, column='id')


        # move the file
        dir = server.get_handoff_dir()
        print("Copied files to handoff dir\n")
        new_pattern = pattern
        file_type = 'main'
        
        # run group checkin
        server.group_checkin(search_key, context, file_path=new_pattern, file_type=file_type, \
                file_range=file_range, mode='copy', info={'type':'2d_plates'})
        
    except:
        server.abort()
        raise
    else:
        server.finish()

if __name__ == '__main__':
    executable = sys.argv[0]
    login = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "l:h", ["login=","help"])
    except getopt.error as msg:
        print(msg)
        sys.exit(2)
    # process options
    for o, a in opts:
        if o in ("-l", "--login"):
            login = a
            print("")
        if o in ("-h", "--help"):
            print("python checkin_plates.py [-l <tactic_login>] <code> <file_range> <file_pattern>")
            print("python checkin_plates.py S0001 1-20 D:/file_dir/plates.####.png")
    if len(args) != 3:
        print("python checkin_plates.py <code> <file_range> <file_pattern>")
        print("python checkin_plates.py S0001 1-20 D:/file_dir/plates.####.png")
        sys.exit(2)
    main(args, login=login)




############################################################
#
#    Copyright (c) 2012, Southpaw Technology
#                        All Rights Reserved
#
#    PROPRIETARY INFORMATION.  This software is proprietary to
#    Southpaw Technology, and is not to be reproduced, transmitted,
#    or disclosed in any way without written permission.
#
#

__all__ = ['PerforceImpl']

import os

from scm_impl import ScmImpl, ScmException

try:
    from P4 import P4, P4Exception
    HAS_PERFORCE = True
except Exception, e:
    HAS_PERFORCE = False
    P4 = None


class PerforceImpl(ScmImpl):

    def __init__(self, **kwargs):
        super(PerforceImpl, self).__init__(**kwargs)

        port = self.kwargs.get("port")
        self.user = self.kwargs.get("user")
        self.depot = self.kwargs.get("depot")
        password = self.kwargs.get("password")
        client = self.kwargs.get("client")

        self.sync_dir = self.kwargs.get("sync_dir")


        if not self.depot:
            self.depot = "depot"


        use_env = False
        if use_env:
            pass
        else:
            # clear the environment to be sure nothing external gets picked up
            if os.environ.has_key("P4USER"):
                del(os.environ['P4USER'])

            if os.environ.has_key("P4PASSWD"):
                del(os.environ['P4PASSWD'])

            if os.environ.has_key("P4PORT"):
                del(os.environ['P4PORT'])

            if os.environ.has_key("P4CLIENT"):
                del(os.environ['P4CLIENT'])


        self.p4 = P4()

        self.p4.user = str(self.user)
        self.p4.port = str(port)
        self.p4.password = str(password)

        if client:
            self.p4.client = str(client)

        self.p4.connect()
        #self.log.append(str(self.p4.run("info")))

        if self.sync_dir:
            self.set_sync_dir(self.sync_dir)

    def ping(self):
        self.log.append("ping")
        return "OK"


    #
    # user specific functions
    #
    def is_logged_in(self):
        self.log.append("p4 login -s")
        try:
            msg = self.p4.run("login", "-s")
        except Exception, e:
            # for some reason, the "p4 login -s" command gives an exception
            # if you are not logged in
            msg = str(e)


        # if a list is returned, then the user is logged in
        if isinstance(msg,list):
            msg = msg[0]
            if isinstance(msg,dict):
                return True


        if msg.find("Your session has expired") != -1:
            return False
        elif msg.find("Perforce password (P4PASSWD) invalid or unset") != -1:
            return False
        elif msg.find("not connected") != -1:
            return False
        else:
            return True





    #
    # depot functions
    #
    def add_depot(self, depot):
        depot = self.p4.fetch_depot(depot)
        self.p4.save_depot(depot)
        return depot

    def get_depots(self):
        depots = self.p4.run("depots")
        return depots


    #
    # branch functions
    #
    def get_branches(self):
        return self.p4.run("branches")



    #
    # workspace functions
    #
    def get_workspaces(self, user=None):
        if not user:
            user = self.user
        if user:
            return self.p4.run("workspaces", "-u", user)
        else:
            return self.p4.run("workspaces")

    def get_workspace_info(self, workspace):
        if not workspace:
            workspace = self.client_name
        return self.p4.run("workspace", "-o", workspace)


    def check_workspace(self):

        workspaces = self.get_workspaces()
        workspaces = [x.get("client") for x in workspaces]

        clientspec = self.p4.fetch_client()
        client_name = clientspec.get("Client")

        self.log.append(workspaces)

        if client_name not in workspaces:
            return False
        else:
            return True



    def set_sync_dir(self, sync_dir):
        '''This function sets the user's workspace.  It allows for dynamic
        setting of user's workspace depending on which directory the user
        is getting the files from.  However, most will have this location
        specifically mapped through //<depot>'''

        ###### DISABLED
        return


        if not sync_dir:
            raise Exception("Sync dir is None")

        self.sync_dir = sync_dir

        # get the client
        clientspec = self.p4.fetch_client()
        clientspec["Client"] = self.client_name
        clientspec["Root"] = self.sync_dir
        clientspec["Owner"] = self.user

        view = []
        view.append("//%s/... //%s/..." % (self.depot, self.client_name) )

        clientspec["View"] = view
        self.p4.save_client(clientspec)


    def add(self, path, changelist="default"):

        if not changelist:
            changelist = "default"


        self.log.append("p4 add -c %s %s" % (changelist, path))
        self.p4.run("add", "-c", changelist, path)


    def checkout(self, repo_dir, recurse=None, depth=None):

        if not repo_dir.startswith("//"):
            repo_dir = "//%s/%s" % (self.depot, repo_dir)
        if not repo_dir.endswith("/..."):
            repo_dir = "%s/..." % repo_dir

        self.log.append("sync -f %s" % repo_dir)
        try:
            #print "sync", "-f", repo_dir
            sync_data = self.p4.run("sync", "-f", repo_dir)
        except Exception, e:
            if str(e).find("up-to-date") == -1:
                raise
            else:
                sync_data = []


        #print "sync: ", sync_data

        data = {}
        error = []
        for entry in sync_data:
            if isinstance(entry, basestring):
                # if the entry is a string, then it is likely an error
                error.append(entry)
            else:
                data[entry.get("clientFile")] = entry.get("action")

        return {
            "value": data,
            "error": error
        }

    def checkout_file(self, repo_path):
        # ????
        self.p4.run("sync", "-f", repo_path)


    def revert(self, sync_path):
        self.log.append("p4 revert %s" % sync_path)
        return self.p4.run("revert", "-a", sync_path)



    def restore(self, sync_path):
        self.log.append("p4 revert %s" % sync_path)
        return self.p4.run("revert", sync_path)


    def edit(self, repo_path, changelist="default"):
        if not changelist:
            changelist = 'default'
        self.log.append("p4 edit -c %s %s" % (changelist, repo_path))
        return self.p4.run("edit", "-c", changelist, repo_path )



    def update(self, repo_path, recurse=None, depth=None):
        pass


    def export(self, repo_path, dst, recurse=None, depth=None):
        pass



    def file_log(self, repo_path):
        self.log.append("p4 filelog %s" % repo_path)
        return self.p4.run("filelog", repo_path )



    def commit_file(self, path, description, keep_editable=False):
        '''Commit a bunch of files'''


        if not path.startswith(self.sync_dir):
            path = "%s/%s" % (self.sync_dir, path)

        status = self.status(path).get(path)

        if status == 'unversioned':
            self.add(path)
        elif status == 'missing':
            self.checkout_file(path)

        elif status == 'modified':
            # not really sure how it can be "modified" without having
            # edited it.
            self.edit(path)
        elif status == 'same':
            self.edit(path)
        elif status == None:
            # if this file is unknown, then add it.
            self.add(path)

        # first create a new changelist
        #changelist = self.add_changelist(description)

        # NOTE: you can only submit individual file to check-in
        # from the default changelist
        self.log.append("p4 submit -r -d \"%s\" \"%s\"" % (description, path))
        if keep_editable:
            ret_val = self.p4.run("submit", "-r", "-d", description, path)
        else:
            ret_val = self.p4.run("submit", "-d", description, path)
        return ret_val


    def commit_changelist(self, changelist="default", description=""):
        keep_open = False       # -r option
        if changelist in [None, 'default']:
            self.log.append("p4 submit -d \"%s\"" % description)
            ret_val = self.p4.run("submit", "-d", description)
        else:
            self.log.append("p4 submit -r -c %s" % changelist)
            ret_val = self.p4.run("submit", "-r", "-c", changelist)
        return ret_val




    def status(self, sync_dir=None):

        if not sync_dir:
            sync_dir = self.sync_dir


        sync_expr = sync_dir
        if sync_expr:
            if os.path.isdir(sync_expr) and not sync_expr.endswith("/..."):
                sync_expr = "%s/..." % sync_expr

        # FIXME: this might be slow on huge repositories

        try:
            if sync_expr:
                self.log.append("p4 diff -sl -f %s"  % sync_expr)
                diff = self.p4.run("diff", "-sl", "-f", sync_expr)
            else:
                self.log.append("p4 diff -sl -f")
                diff = self.p4.run("diff", "-sl", "-f")

        except Exception, e:
            # if there is an exception, likely this is because there are 
            # no files on the client and Perforce does not like this
            print "WARNING: ", e
            print self.p4.run("info")
            diff = []


        info = {}
        for entry in diff:
            client_file = entry.get("clientFile")
            client_file = client_file.replace("\\", "/")
            if not client_file.startswith(sync_dir):
                client_file = "%s/%s" % (sync_dir, client_file)
            status = entry.get("status")
            if status == 'diff':
                info[client_file] = 'modified'
            else:
                info[client_file] = status


        # find opened files
        opened_files = self.p4.run("opened")
        new_files = {} 
        for opened_file in opened_files:
            if opened_file['action'] != 'add':
                continue
            path = opened_file['depotFile']
            path = path.replace("//%s" % self.depot, self.sync_dir)
            new_files[path] = 'add'

        for root, dirnames, basenames in os.walk(sync_dir):
            for dirname in dirnames:
                client_path = "%s/%s" % (root, dirname)
                client_path = "%s/%s" % (root, dirname)
                client_path = client_path.replace("\\", "/")

                if client_path.find("/.tactic") != -1:
                    continue

                info[client_path] = 'directory'

            for basename in basenames:
                client_path = "%s/%s" % (root, basename)
                client_path = client_path.replace("\\", "/")

                # ignore .tactic directory
                if client_path.find("/.tactic") != -1:
                    continue

                status = info.get(client_path)

                if new_files.get(client_path):
                    info[client_path] = 'added'
                elif not status:
                    info[client_path] = 'unversioned'
                elif status == 'same' and os.access(client_path, os.W_OK):
                    info[client_path] = 'editable'


        return info




    def add_changelist(self, description, changelist=None):
        changespec = self.p4.fetch_change()
        changespec['Description'] = description
        changespec['User'] = self.user
        info = self.p4.save_change(changespec)
        return info


    def get_changelist_files(self, changelist='default'):
        files = self.p4.run("opened", "-c", changelist)
        return files

    def get_changelist_info(self, changelist='default'):
        info = self.p4.run("describe", changelist)
        return info


    def get_changelists_by_user(self, user=None):
        if not user:
            user = self.user
        changeslists = self.p4.run("changelists", "-u", user, "-s", "pending", "-c", self.client_name)
        return changeslists




    def get_changelists(self, counter='tactic'):
        changeslists = self.p4.run("review", "-t", counter)
        return changeslists


    def set_changelist_counter(self, changelist, counter='tactic'):
        changeslists = self.p4.run("review", "-t", counter, '-c', changelist)
        return changeslists





if __name__ == '__main__':
    scm = PerforceImpl()
    #print "branches: ", svn.get_all_branches()
    #print "tags: ", svn.get_all_tags("3.6")

    repo_dir = "src/main/documents"
    scm.checkout(repo_dir)









###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['MayaModelValidate', 'MayaModelCheckin']

try:
    import maya.cmds as cmds
except ImportError as e:
    #print "WARNING: examples.py requires Maya python libraries"
    pass

from .handler import Handler



class MayaModelValidate(Handler):
    def execute(self):

        # get the search key from the delivered package
        search_key = self.get_package_value("search_key")

        # get the sobject from the server
        sobject = self.server.get_by_search_key(search_key)
        if not sobject:
            raise Exception("SObject with search key [%s] does not exist" % \
                search_key)

        # code and verify in maya that the node is in session
        code = sobject.get('code')
        if not cmds.ls(code):
            raise Exception("Cannot checkin: [%s] does not exist" % code)

        self.set_output_value('sobject', sobject)


    def undo(self):
        # nothing to undo in session
        pass




class MayaModelCheckin(Handler):

    def execute(self):

        # get the sobject passed in
        sobject = self.get_input_value('sobject')
        code = sobject.get('code')
        search_key = self.get_package_value("search_key")

        # get the designated local directory to put temporary files
        tmp_dir = self.get_package_value("local_dir")
        path = "%s/%s.ma" % (tmp_dir, code)

        context = self.get_package_value("asset_context")
        # FIXME: ignore subcontext for now
        #subcontext = self.get_package_value("asset_sub_context")
        #if subcontext:
        #    context = "%s/%s" % (context, subcontext)

        # save out the file
        cmds.file( rename=path)
        cmds.file( save=True, type='mayaAscii')

        # checkin the file that was just saved
        self.server.upload_file(path)
        snapshot = self.server.simple_checkin(search_key, context, path)

        # add a mock dependency
        snapshot_code = snapshot.get("code")
        self.server.add_dependency(snapshot_code, "C:/tt.pdf")


 
    def undo(self):
        # nothing to undo in session
        pass








###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys

from tactic_client_lib import TacticServerStub


def main(args):
    usage = "USAGE: checkin.py <search_type> <code> [context] <path>\n"
    usage += "example: python checkin.py beat Sc01.Bt01 .\\test\\image.png"

    context = "publish"

    # TODO: lots of assumptions here
    if len(args) == 2:
        # assume code and file path are equivalent
        code = args[1]
        file_path = args[1]
    elif len(args) == 3:
        code = args[1]
        file_path = args[2]
    elif len(args) == 4:
        code = args[1]
        context = args[2]
        file_path = args[3]
    else:
        print(usage)
        return

    search_type = args[0]

    server = TacticServerStub()

    # do the actual work
    server.start("Checked in file [%s] to [%s] - [%s]" % (file_path, search_type, code) )
    try:
        # query all of the search_types to simplify argument
        if search_type.find("/") == -1:
            columns = ["search_type"]
            results = server.query("sthpw/search_object", columns=columns)
            for result in results:
                test = result.get("search_type")
                if test.endswith("/%s" % search_type):
                    search_type = test
                    break
            else:
                raise Exception("Search type [%s] not found" % search_type)

        search_key = server.build_search_key(search_type, code)

        # upload the file
        server.upload_file(file_path)

        # checkin the uploaded file
        result = server.simple_checkin(search_key, context, file_path)
    except Exception as e:
        server.abort()
        print("ERROR: %s" % e)
    else:
        server.finish()


if __name__ == '__main__':
    
    executable = sys.argv[0]
    args = sys.argv[1:]
    main(args)




############################################################
#
#    Copyright (c) 2012, Southpaw Technology
#                        All Rights Reserved
#
#    PROPRIETARY INFORMATION.  This software is proprietary to
#    Southpaw Technology, and is not to be reproduced, transmitted,
#    or disclosed in any way without written permission.
#
#

import unittest
import getpass
import os
import shutil

from subversion import *


class ScmTest(unittest.TestCase):

    def test_all(self):

        user = 'remko'
        password = getpass.getpass("Password: ")

        self.scm = Subversion(user=user, password=password)
        self.scm.set_root("file:///home/apache/test_svn_repo")


        self._test_empty_checkout()
        self._test_checkout_single_file()
        self._test_delivery()




    def _test_empty_checkout(self):
        #repo_path = "sound/sound.wav"
        repo_path = "sound.wav"
        sync_dir = "test_empty"

        branch = "1.1"
        self.scm.set_branch(branch)
        self.scm.set_sync_dir(sync_dir)

        repo_dir = os.path.dirname(repo_path)
        self.scm.checkout(repo_dir, sync_dir, depth="empty")

        self.assertEquals( False, os.path.exists(repo_path) )
        shutil.rmtree(sync_dir)



    def _test_checkout_single_file(self):

        # set the sync dir
        sync_dir = "test_checkout_single"
        self.scm.set_sync_dir(sync_dir)
        self.scm.set_branch("1.1")

        #repo_path = "sound/sound.wav"
        repo_path = "sound.wav"
        sync_path = "%s/%s" % (sync_dir, os.path.basename(repo_path))
        repo_dir = os.path.dirname(repo_path)

        # test checkout of a single file (trick)
        self.scm.checkout_file(repo_path)

        self.assertEquals( True, os.path.exists(sync_path) )
        shutil.rmtree(sync_dir)





    def _test_checkout(self):

        # set the sync dir
        sync_dir = "test_checkout"
        self.scm.set_sync_dir(sync_dir)
        self.scm.set_branch("1.1")

        #repo_path = "sound/sound.wav"
        repo_path = "sound.wav"
        sync_path = "%s/%s" % (sync_dir, os.path.basename(repo_path))
        repo_dir = os.path.dirname(repo_path)

        # test checkout of a single file (trick)
        self.scm.checkout(repo_dir, sync_dir, depth="empty")
        self.scm.export(repo_path, sync_path)

        self.assertEquals( True, os.path.exists(sync_path) )

        shutil.rmtree(sync_dir)






    def _test_delivery(self):
        '''deliver a file to the repository'''
        branch = "1.1"
        #repo_path = "docs/documents2.txt"
        repo_path = "documents2.txt"
        sync_dir = "test_delivery"


        src_path = "./document2.txt"
        f = open(src_path, 'w')
        f.write("This is a document")
        f.close()



        self.scm.set_branch(branch)

        self.scm.set_sync_dir(sync_dir)
        self.scm.deliver_file(src_path, repo_path)


        shutil.rmtree(sync_dir)




if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python 
###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['Sample3dTest']

import tacticenv
from tactic_client_lib import TacticServerStub

import unittest
import xmlrpclib, sys, os, shutil

class Sample3dTest(unittest.TestCase):

    def setUp(self):
        pass

    def test_all(self):
        print "Running Sample3d Test"

        from pyasm.security import Batch
        from pyasm.biz import Project
        Batch()
        Project.set_project("sample3d")


        #self.server = TacticServerStub(protocol="local")
        self.server = TacticServerStub(protocol="xmlrpc")
        project_code = "sample3d"
        self.server.set_project(project_code)


        self.server.start("Sample3d Test")
        try:
            self._test_create_search_type()
            self._test_create_submission()
            self._test_get_submission()
            self._test_shot_sequence_hierarchy()
            self._test_query_snapshots()
            #self._test_performance()
        except Exception:
            self.server.abort()
            raise
        self.server.abort()


        #try:
        #    self.server.query("prod/asset")
        #except Exception:
        #    self.server.abort()
        #    raise
        #self.server.abort()



    def _test_query_snapshots(self):
        filters = []
        filters.append( ['context', 'model'] )
        filters.append( ['search_type', 'prod/asset?project=sample3d'] )
        snapshots = self.server.query_snapshots(filters=filters, include_paths=True)

        import time
        start = time.time()
        for snapshot in snapshots:
            print snapshot.get('__search_key__')
            print snapshot.get('__paths__')
            print "parent: ", snapshot.get('__parent__')
        print time.time() - start




    def _test_create_search_type(self):
        search_type = 'test'
        search_type_obj = self.server.create_search_type(search_type)
        print search_type_obj




    def _test_performance(self):

        for i in range(0,1):
            assets = self.server.query("prod/asset")
            for asset in assets:
                asset_key = asset.get("__search_key__")
                snapshots = self.server.get_all_children(asset_key,'sthpw/snapshot')
                #snapshot = self.server.get_snapshot(asset_key,context='model', include_paths=True)
                #print snapshot.get('__paths__')


        




    def _test_get_submission(self):

        server = TacticServerStub()
        server.set_project("sample3d")

        # choose some arbitrary bin
        bin_id = 4
        filters = []
        filters.append( ['bin_id', bin_id] )
        connectors = server.query("prod/submission_in_bin", filters)

        # get all of the submissions from the bin
        submission_ids = [x.get('submission_id') for x in connectors]
        filters = [ ['id', submission_ids] ]
        submissions = server.query("prod/submission", filters)


        # get all of the snapshots from the submissions
        for submission in submissions:
            search_key = submission.get('__search_key__')

            print "-"*20
            snapshot = server.get_snapshot(search_key, include_paths=True)

            paths = snapshot.get('__paths__')
            for path in paths:
                print path



    def _test_create_submission(self):

        server = TacticServerStub()
        server.set_project("sample3d")

        # choose some arbitrary bin
        bin_id = 4
        filters = []


        # asset
        parent_type = "prod/asset"
        parent_code = "chr001"
        parent_key = server.build_search_key(parent_type, parent_code)
        parent = server.get_by_search_key(parent_key)
        parent_id = parent.get('id')

        # create a submission
        data = {
            'description': 'A test submission',
            'artist': 'joe',
            'context': 'model'
        }
        submission = server.insert("prod/submission", data, parent_key=parent_key)
        submission_key = submission.get('__search_key__')
        submission_id = submission.get('id')

        file_path = './miso_ramen.jpg'
        context = "publish"
        snapshot = server.simple_checkin(submission_key, context, file_path, mode="upload")

        # no connect to the bin with a connector
        data = {
            "bin_id": bin_id,
            'submission_id': submission_id
        }
        server.insert("prod/submission_in_bin", data)




    def _test_shot_sequence_hierarchy(self):
        shot_key = "prod/shot?project=sample3d&code=RC_001_001"
        shot = self.server.get_by_search_key(shot_key)
        
        parent = self.server.get_parent(shot_key)
        self.assertEquals("RC_001", parent.get("code") )



if __name__ == "__main__":
    unittest.main()





###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys

from tactic_client_lib import TacticServerStub

SEARCH_TYPE = "prod/shot"

def main(args):
    # USAGE: query_shot.py <shot_code>
    shot_code = args[0]

    server = TacticServerStub()
    search_key = server.build_search_type(SEARCH_TYPE)

    # do the actual work
    server.start("Queried shot [%s]" % shot_code)
    try:
        filters = [
            ('code', shot_code)
        ]
        print server.query(search_key, filters)

    except:
        server.abort()
        raise
    else:
        server.finish()



if __name__ == '__main__':
    
    executable = sys.argv[0]
    args = sys.argv[1:]
    main(args)




#!/usr/bin/python 
###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import unittest, sys

# import the client lib
sys.path.insert( 0, ".." )
from tactic_client_lib.interpreter import *



class PipelineTest(unittest.TestCase):

    def test_all(self):
        
        self.test_handler = 'tactic_client_lib.test.TestHandler'
        self.pipeline_xml = '''
        <pipeline>
          <process name='model'>
            <action class='%s'>
                <test>pig</test>
                <test2>cow</test2>
            </action>
          </process>
          <process name='texture'>
            <action class='%s'/>
          </process>
          <process name='rig'>
            <action class='tactic_client_lib.test.TestNextProcessHandler'/>
          </process>
          <process name='extra'/>
          <process name='extra2'/>
          <connect from='model' to='texture'/>
          <connect from='model' to='rig'/>
          <connect from='rig' to='publish'/>
        </pipeline>
        ''' % (self.test_handler, self.test_handler)

        self.pipeline = Pipeline(self.pipeline_xml)

        self._test_pipeline()
        self._test_interpreter()


    def _test_pipeline(self):
        # get the output names
        output_names = self.pipeline.get_output_process_names('model')
        self.assertEquals( ['texture', 'rig'], output_names )

        # get the input names
        input_names = self.pipeline.get_input_process_names('texture')
        self.assertEquals( ['model'], input_names)

        # get the handler class of model
        handler_class = self.pipeline.get_handler_class('model')
        self.assertEquals( self.test_handler, handler_class)


        # small test running through pipeline
        process = self.pipeline.get_first_process_name()
        self.assertEquals( 'model', process)


    def _test_interpreter(self):

        # create a package to be delivered to each handler
        package = {
            'company': 'Acme',
            'city': 'Toronto',
            'context': 'whatever'
        }

        # use client api
        from tactic_client_lib import TacticServerStub
        server = TacticServerStub()

        interpreter = PipelineInterpreter(self.pipeline_xml)
        interpreter.set_server(server)
        interpreter.set_package(package)
        interpreter.execute()

        # introspect the interpreter to see if everything ran well
        handlers = interpreter.get_handlers()
        process_names = [x.get_process_name() for x in handlers]
        expected = ['model', 'texture', 'rig', 'extra1', 'extra2']
        self.assertEquals( expected, process_names )

        # make sure all the handlers completed
        self.assertEquals( 5, len(handlers) )
        for handler in handlers:
            self.assertEquals( "complete", handler.get_status() )

            # check that the package is delivered to the input
            self.assertEquals("Acme", handler.get_input_value('company') )
            self.assertEquals("Toronto", handler.get_input_value('city') )

            process_name = handler.get_process_name()
            if process_name == 'model':
                self.assertEquals("Acme", handler.company)

                self.assertEquals("pig", handler.get_option_value('test') )
                self.assertEquals("cow", handler.get_option_value('test2') )

            # ensure input settings propogate
            if process_name == 'extra1':
                self.assertEquals("test.txt", handler.get_output_value('file'))
                self.assertEquals("Acme", handler.get_package_value('company'))





if __name__ == "__main__":
    unittest.main()




###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['Session']

import os, sys, urllib, xmlrpclib
from xml.dom.minidom import parseString
from xml.dom.minidom import getDOMImplementation

from tactic_node_util import TacticNodeUtil
from node_data import NodeData
from application import Application


class Session(object):
    '''helper class to introspect the session'''

    def __init__(self):
        self.xml = None
        self.doc = None
        self.root = None

        from tactic_client_lib import TacticServerStub
        self.server = TacticServerStub.get()

        # TODO: pull this out of here!!!
        self.util = TacticNodeUtil()
        self.app = Application.get()

    def introspect(self):
        '''introspect the session and create a session xml from it'''
        # create an xml document
        xml_impl = getDOMImplementation()
        self.doc = xml_impl.createDocument(None, "session", None)
        self.root = self.doc.documentElement

        # go through the tactic
        tactic_nodes = self.util.get_all_tactic_nodes() 
        tactic_nodes.sort()
        for tactic_node in tactic_nodes:

            node_data = NodeData(tactic_node)
            ref_node = node_data.get_ref_node()

            # set some more info on the ref node
            ref_node.setAttribute("tactic_node", tactic_node)
            self.root.appendChild(ref_node)

        self.xml = self.doc.toprettyxml()
        return self.xml


    def get_last(self, pid=None):
        '''Get the last sesson recorded from the last session.  This is useful
        for applications that do not have direct access to the application and
        must go through the database to get this information'''

        # get more info
        if not pid:
            pid = os.getpid()
        login = self.server.get_login()

        search_type = "prod/session_contents"
        filters = []
        filters.append( ["pid", pid] )
        filters.append( ["login", login] )
        limit = 1
        order_bys = ['timestamp desc']
        sessions = self.server.query(search_type, filters, order_bys=order_bys, limit=limit)

        if sessions:
            session = sessions[0]
            self.xml = session.get('session')

            self.doc = parseString(self.xml)
            self.root = self.doc.documentElement

            return sessions
        else:
            self.doc = None
            self.root = None
            self.xml = None
            return None




    def get_snapshots(self):
        '''gets the snapshots that are in the session'''

        # create an xml document
        self.root = self.doc.documentElement

        snapshot_codes = []
        for node in self.root.childNodes:
            if node.nodeName != "ref":
                continue
            snapshot_code = node.getAttribute("snapshot_code")
            snapshot_codes.append(snapshot_code)

        filters = []
        filters.append( ['code', snapshot_codes] )

        snapshots = self.server.query("sthpw/snapshot", filters )
        return snapshots





    def commit(self, xml=None):
        '''commit this to the database'''

        if not xml:
            xml = self.xml

        # get more info
        pid = os.getpid()
        login = self.server.get_login()

        #data = { 'pid': pid, 'login': login, 'data': xml }
        #self.server.insert( search_type, data)

        self.server.commit_session(xml, pid)
        










#!/usr/bin/env python

###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

import sys, os, getpass

try:
    import tacticenv
except ImportError:
    pass

from tactic_client_lib import TacticServerStub


def main(args):

    server = TacticServerStub(setup=False)
    server.get_info_from_user(force=True)
    return

    """
    old_server_name = server.server_name
    old_project_code = server.project_code
    default_login = getpass.getuser()

    print
    server_name = raw_input("Enter name of TACTIC server (%s): " % old_server_name)
    if not server_name:
        server_name = old_server_name

    print
    login = raw_input("Enter user name (%s): " % default_login)
    if not login:
        login = default_login

    print
    password = getpass.getpass("Enter password: ")

    print
    project_code = raw_input("Project (%s): " % old_project_code)
    if not project_code:
        project_code = old_project_code

    server.set_server(server_name)


    # commit info to a file
    if os.name == "nt":
        dir = "C:/sthpw/etc"
    else:
        dir = "/tmp/sthpw/etc"
    if not os.path.exists(dir):
        os.makedirs(dir)

    filename = "%s.tacticrc" % login
    path = "%s/%s" % (dir,filename)

    # do the actual work
    ticket = server.get_ticket(login, password)
    print "Got ticket [%s] for [%s]" % (ticket, login)

    file = open(path, 'w')
    file.write("login=%s\n" % login)
    file.write("server=%s\n" % server_name)
    file.write("ticket=%s\n" % ticket)
    if project_code:
        file.write("project=%s\n" % project_code)

    file.close()
    print
    print "Saved to [%s]" % path
    """



if __name__ == '__main__':
    
    executable = sys.argv[0]
    args = sys.argv[1:]
    main(args)




import tacticenv
from tactic_client_lib import TacticServerStub

server = TacticServerStub(protocol="xmlrpc")
#server.set_project("sample3d")
server.set_ticket("60128265ebea63620de1f5b0ffc502eb")
search_type = "prod/shot"

server.start(title='cow')
sobject = server.insert(search_type, { 'description': 'wow'} )
server.update(search_key, { 'description': 'another update'} )
server.finish()

sobject = server.insert(search_type, { 'description': 'wow2'} )
sobject = server.insert(search_type, { 'description': 'wow3'} )






import sys, os, shutil, getopt

from tactic_client_lib import TacticServerStub

SEARCH_TYPE = "prod/render"

def move_file(file_paths, new_dir):
    '''move file to the handoff dir'''
    new_file_paths = []
    for file_path in file_paths:
        file_name = os.path.basename(file_path)
        new_file_path = '%s/%s' %(new_dir, file_name)
        shutil.move(file_path, new_file_path)
        '''
        while not os.path.exists(new_file_path):
            sys.stdout.write('.')
        '''
        new_file_paths.append(new_file_path)
    return new_file_paths

def expand_paths( file_path, file_range ):
    '''expands the file paths, replacing # as specified in the file_range'''
    file_paths = []
    
    # frame_by is not really used here yet
    frame_by = 1
    if file_range.find("/") != -1:
        file_range, frame_by = file_range.split("/")
    frame_start, frame_end = file_range.split("-")
    frame_start = int(frame_start)
    frame_end = int(frame_end)
    frame_by = int(frame_by)

    # find out the number of #'s in the path
    padding = len( file_path[file_path.index('#'):file_path.rindex('#')] )+1

    for i in range(frame_start, frame_end+1, frame_by):
        expanded = file_path.replace( '#'*padding, str(i).zfill(padding) )
        file_paths.append(expanded)

    return file_paths

    

def main(args, login=None):
    # USAGE: checkin_render_layer.py   
    shot_code = args[0]
    layer_name = args[1]
    version = args[2]
    context = args[3]
    file_range = args[4]
    pattern = args[5]

    server = TacticServerStub(login)

    # do the actual work
    server.start("Checked in file group [%s] to shot [%s] layer [%s]" % (pattern, shot_code, layer_name))
    try:
        # move the file
        dir = server.get_handoff_dir()
        paths = expand_paths(pattern, file_range)
        move_file(paths, dir)

        file_name = os.path.basename(pattern)
        new_pattern = '%s/%s' % (dir, file_name)
        print("Files moved to handoff dir.\n")
        
        # checkin the moved files  
        filters = []
        filters.append(('shot_code', shot_code))
        filters.append(('name', layer_name))
        results = server.query('prod/layer', filters)
        
        # take the first one
        if results:
            id = results[0].get('id')

        search_type = server.build_search_type('prod/layer')
        # find the layer snapshot
        filters = []
        filters.append(('version', version))
        filters.append(('search_type', search_type))
        filters.append(('search_id', id))
        #TODO : may need a context to search for the proper layer
        results = server.query('sthpw/snapshot', filters)
        snap_code = ''
        if results:
            snap_code = results[0].get('code')
        
        # find the render
        render = None
        filters = []
        filters.append(('search_type', search_type))
        filters.append(('search_id', id))
        filters.append(('snapshot_code', snap_code))
        results = server.query(SEARCH_TYPE, filters)
        if results:
            render = results[0]

        if not render:
            render_data = {
                'search_type': search_type,
                'search_id': id,
                'snapshot_code': snap_code
            }

            render = server.insert("prod/render", render_data)
        '''
        results = server.query(SEARCH_TYPE, filters)
        render_id = 0
        if results:
            render_id = results[0].get('id')
        # find the render id    
        search_key = server.build_search_key(SEARCH_TYPE, render_id, column='id')
        '''
        file_type = 'main'
        
        # run group checkin
        server.group_checkin(render.get("__search_key__"), context=context, file_path=new_pattern, file_type=file_type, file_range=file_range)
        
    except:
        server.abort()
        raise
    else:
        server.finish()

if __name__ == '__main__':
    executable = sys.argv[0]
    #args = sys.argv[1:]
    login = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "l:h", ["login=","help"])
    except getopt.error as msg:
        print(msg)
        sys.exit(2)
    # process options
    for o, a in opts:
        if o in ("-l", "--login"):
            login = a
        if o in ("-h", "--help"):
            print("python checkin_render_layer.py <shot_code> <layer_name> <version> <context> <file_range> <file_pattern>")
            print("python checkin_render_layer.py S0001 layer1 1 lgt 1-20 D:/file_dir/plates.####.png")
            sys.exit(0)
    print(args, len(args))
    if len(args) != 6:
        print("python checkin_render_layer.py <shot_code> <layer_name> <version> <context> <file_range> <file_pattern>")
        print("python checkin_render_layer.py S0001 layer1 1 lgt 1-20 D:/file_dir/plates.####.png")
        sys.exit(2)
    main(args, login=login)




import sys, os, shutil, getopt

from tactic_client_lib import TacticServerStub

SEARCH_TYPE = "prod/shot"

def copy_file(file_paths, new_dir):
    '''copy file to the handoff dir'''
    new_file_paths = []
    for file_path in file_paths:
        file_name = os.path.basename(file_path)
        new_file_path = '%s/%s' %(new_dir, file_name)
        #shutil.move(file_path, new_file_path)
        shutil.copy(file_path, new_file_path)
        print(new_file_path)
        if not os.path.exists(new_file_path):
            print("[%s] does not exist!!!" % new_file_path)
        new_file_paths.append(new_file_path)
    return new_file_paths

def expand_paths( file_path, file_range ):
    '''expands the file paths, replacing # as specified in the file_range'''
    file_paths = []
    
    # frame_by is not really used here yet
    frame_by = 1
    if file_range.find("/") != -1:
        file_range, frame_by = file_range.split("/")
    frame_start, frame_end = file_range.split("-")
    frame_start = int(frame_start)
    frame_end = int(frame_end)
    frame_by = int(frame_by)

    # find out the number of #'s in the path
    padding = len( file_path[file_path.index('#'):file_path.rindex('#')] )+1

    for i in range(frame_start, frame_end+1, frame_by):
        expanded = file_path.replace( '#'*padding, str(i).zfill(padding) )
        file_paths.append(expanded)

    return file_paths

    

def main(args, login=None):
    # USAGE: checkin_render.py   
    type = args[0]
    if type == "shot":
        parent_search_type = "prod/shot"
    elif type == "asset":
        parent_search_type = "prod/asset"
    

    else:
        parent_search_type = type

    code = args[1]
    file_range = args[2]
    pattern = args[3]
    layer_name = ''
    context = 'publish'

    if type == "layer":
        parent_search_type = "prod/layer"
        code, layer_name = args[1].split('|')
        
    server = TacticServerStub(login)

    # do the actual work
    server.start("Checked in file group [%s] to %s [%s]" % (pattern,type,code))

    try:
        # checkin the uploaded file  
        filters = []
        if type=='layer':
            filters.append(('shot_code', code))
            filters.append(('name', layer_name))
        else:
            filters.append(('code', code))
        results = server.query(parent_search_type, filters)
       
        # take the first one
        if results:
            id = results[0].get('id')
        else:
            if type =='layer':
                print("%s Code [%s] name [%s] not found. Please insert an entry in TACTIC first." % (type, code, layer_name))
            else:
                print("%s Code [%s] not found. Please insert an entry in TACTIC first." % (type, code))
        search_type = server.build_search_type(parent_search_type)
        file_type = 'main'
        render_type = '' # not used yet
        # move the file
        dir = server.get_handoff_dir()
        paths = expand_paths(pattern, file_range)
        copy_file(paths, dir)

        file_name = os.path.basename(pattern)
        new_pattern =  '%s/%s' %(dir, file_name)
        print("Copied files to handoff dir\n")
        render = find_render(server, search_type, id, login, render_type)
        if not render:
            render_data = {
                'search_type': search_type,
                'search_id': id,
                'login': login
                #'type': render_type
            }
            render = server.insert("prod/render", render_data)
        
        # run group checkin
        server.group_checkin(render.get("__search_key__"), context=context, file_path=new_pattern, file_type=file_type, file_range=file_range)
        
    except:
        server.abort()
        raise
    else:
        server.finish()


def find_render(server, search_type, id, login, type):
    render = None
    filters = []
    filters.append(('search_type', search_type))
    filters.append(('search_id', id))
    filters.append(('login', login))
    #filters.append(('type', type))
    results = server.query('prod/render', filters)
    if results:
        render = results[0]
    return render

if __name__ == '__main__':
    
    executable = sys.argv[0]
    login = None
    try:
        opts, args = getopt.getopt(sys.argv[1:], "l:h", ["login=","help"])
    except getopt.error as msg:
        print(msg)
        sys.exit(2)
    # process options
    for o, a in opts:
        if o in ("-l", "--login"):
            login = a
            print("")
        if o in ("-h", "--help"):
            print("python checkin_render.py -l <tactic_login> <type> <code> <file_range> <file_pattern>")
            print("python checkin_render.py -l admin shot S0001 1-20 D:/file_dir/plates.####.png")
    if not login:
        print("-l <tactic_login> is required for this script")
        print("python checkin_render.py -l <tactic_login> <type> <code> <file_range> <file_pattern>")
        sys.exit(2)
    if len(args) != 4:
        print("python checkin_render.py -l <tactic_login> <type> <code> <file_range> <file_pattern>")
        print("python checkin_render.py -l admin shot S0001 1-20 D:/file_dir/render.####.png")
        print('python checkin_render.py -l admin layer "S0001|diffuse_layer" 1-20 D:/file_dir/render.####.png')
        sys.exit(2)
    main(args, login=login)



############################################################
#
#    Copyright (c) 2012, Southpaw Technology
#                        All Rights Reserved
#
#    PROPRIETARY INFORMATION.  This software is proprietary to
#    Southpaw Technology, and is not to be reproduced, transmitted,
#    or disclosed in any way without written permission.
#
#

from delegate import *
from subversion import *
from perforce import *




###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

_all__ = ["NodeData"]

from xml.dom.minidom import parseString
from xml.dom.minidom import getDOMImplementation



class NodeData(object):
    '''class which stores tactic specific data on the node'''
    ATTR_NAME = "tacticNodeData"
    ATTR_NAME2 = "notes"

    def __init__(self, app_node_name):
        self.app_node_name = app_node_name

        self.data = {}

        from application import Application
        self.app = Application.get()

        self.init()


    def init(self):
        xml = self.app.get_attr(self.app_node_name, self.ATTR_NAME)
        if xml == "":
            xml = "<node/>"

        # FIXME: this is for XSI
        xml = xml.replace("\\n", "\n")

        self.dom = None
        try:
            self.dom = parseString(xml)

        except Exception, e:
            print "Warning: node '%s' has invalid tacticNodeData" % self.app_node_name
            self.clear()
            

    
    def clear(self):
        '''clears the dom by creating an empty one'''
        xml = "<node/>"
        self.dom = parseString(xml)

    def add_node(self, node_name):
        root = self.dom.documentElement
        node = self.dom.createElement(node_name)
        root.appendChild(node)
        return node
        

    def _get_node(self, node_name):
        root = self.dom.documentElement
        nodes = root.childNodes
        for node in nodes:
            if node.__class__.__name__ == "Text":
                continue
            if node.nodeName == node_name:
                return node


    def get_attr(self, node_name, attr):
        node = self._get_node(node_name)
        if not node:
            return ""
        value = node.getAttribute(attr)
        if not value:
            return ""
        else:
            return value


    def set_attr(self, node_name, attr, value):
        node = self._get_node(node_name)
        if not node:
            node = self.add_node(node_name)
        node.setAttribute(attr,value)


    def commit(self):
        xml = self.dom.toxml()
        xml = xml.replace("\n", "\\n")
        xml = xml.replace('"', "'")

        self.app.add_attr(self.app_node_name, self.ATTR_NAME, "string")
        self.app.add_attr(self.app_node_name, self.ATTR_NAME2, "string")
        self.app.set_attr(self.app_node_name, self.ATTR_NAME, xml, "string" )
        self.app.set_attr(self.app_node_name, self.ATTR_NAME2, xml , "string")



    def dump(self):
        print self.dom.toxml()

   
    def create(self):
        '''create the necessary attributes if they do not exists'''
        if not self.app.attr_exists(self.app_node_name, self.ATTR_NAME):
            self.app.add_attr(self.app_node_name, self.ATTR_NAME, type="string")
            
        if not self.app.attr_exists(self.app_node_name, self.ATTR_NAME2):
            self.app.add_attr(self.app_node_name, self.ATTR_NAME2, type="string")


        # initialize the data
        self.init()



    def get_ref_node(self):
        node = self._get_node("ref")
        return node


        






###########################################################
#
# Copyright (c) 2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

'''query.py: simple script to query for set information
'''


# import the client api library
from tactic_client_lib import TacticServerStub

def main():
    # get an instance of the stub
    server = TacticServerStub()

    # start the transaction
    server.start("Set query")
    try:

        # define the search type we are searching for
        search_type = "prod/asset"
          
        # define a filter
        filters = [] 
        filters.append( ("asset_library", "chr") )
          
        # do the query
        assets = server.query(search_type, filters)

        # show number found
        print("found [%s] assets" % len(assets) )
          
        # go through the asset and print the code
        for asset in assets:
            code = asset.get("code")  
            print(code)


    except:
        # in the case of an exception, abort all of the interactions
        server.abort()
        raise
    else:
        # otherwise, finish the transaction
        server.finish()


if __name__ == '__main__':
    main() 





#!/usr/bin/env python

import codecs

from setuptools import setup, find_packages

version = '4.8'

setup(name='tactic_client_lib',
      version=version,
      install_requires=['six'],
      description='Tactic Client Library',
      author='Roland van Laar',
      author_email='roland@micite.net',
      url='https://github.com/rvanlaar/tactic-client',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      scripts=['bin/get_ticket.py', 'bin/create_project.py'],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.7'
      ]
     )
